<?php

/**
* The dashboard-specific functionality of the plugin.
*
* @link 		https://entusiastics.com
* @since 		1.0.0
*
* @package 	Entusiastics_Quiz_Game
* @subpackage 	Entusiastics_Quiz_Game/admin
*/

class Entusiastics_Quiz_Game_Admin {

	/**
	 * The plugin options.
	 *
	 * @since 		1.0.0
	 * @access 		private
	 * @var 		string 			$options    The plugin options.
	 */
	private $options;

	/**
	 * The ID of this plugin.
	 *
	 * @since 		1.0.0
	 * @access 		private
	 * @var 		string 			$plugin_name 		The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since 		1.0.0
	 * @access 		private
	 * @var 		string 			$version 			The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since 		1.0.0
	 * @param 		string 			$Quiz_Game 		The name of this plugin.
	 * @param 		string 			$version 			The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;

		define( 'ENTUSIASTICS_QUIZ_GAME_ADMIN_PATH', plugin_dir_url( __FILE__ ));

		$this->set_options();

	}

	/**
     * Adds notices for the admin to display.
     * Saves them in a temporary plugin option.
     * This method is called on plugin activation, so its needs to be static.
     */
	public static function add_admin_notices() {

		$notices = get_option( 'entusiastics_quiz_game_deferred_admin_notices', array() );

		apply_filters( 'entusiastics_quiz_game_admin_notices', $notices );
		update_option( 'entusiastics_quiz_game_deferred_admin_notices', $notices );

    } // add_admin_notices

	/**
	 * Adds a settings page link to a menu
	 *
	 * @link 		https://codex.wordpress.org/Administration_Menus
	 * @since 		1.0.0
	 * @return 		void
	 */
	public function add_menu() {

		$entusiastics_caps = current_user_can('manage_options') ? 'manage_options' : 'entusiastics_manage';

		// Top-level page
		// add_menu_page( $page_title, $menu_title, $capability, $menu_slug, $function, $icon_url, $position );

		// Submenu Page
		// add_submenu_page( $parent_slug, $page_title, $menu_title, $capability, $menu_slug, $function);

		add_menu_page(
			esc_html__('Entusiastics - Play', 'entusiastics-quiz-game'),
			esc_html__('Entusiastics - Play', 'entusiastics-quiz-game'),
			$entusiastics_caps,
			$this->plugin_name . '-quizzes',
			array( $this, 'page_landing' ),
			'dashicons-forms',
		);

		$hook_quizzes = add_submenu_page(
			$this->plugin_name . '-quizzes',
			apply_filters( $this->plugin_name . '-quizzes-list-page-title', esc_html__( 'Quizzes', 'entusiastics-quiz-game' ) ),
			apply_filters( $this->plugin_name . '-quizzes-list-menu-title', esc_html__( 'Quizzes', 'entusiastics-quiz-game' ) ),
			$entusiastics_caps,
			$this->plugin_name . '-quizzes-list',
			array( 'QuizzesTableController', 'render_quizzes_list' )
		);

		add_action( "load-$hook_quizzes", array( $this, 'quizzes_list_page_add_option') );

		add_submenu_page(
			NULL,
			apply_filters( $this->plugin_name . '-quizz-page-title', esc_html__( 'Nuevo Quizz', 'entusiastics-quiz-game' ) ),
			apply_filters( $this->plugin_name . '-quizz-menu-title', esc_html__( 'Añadir/Editar Nuevo Quizz', 'entusiastics-quiz-game' ) ),
			$entusiastics_caps,
			$this->plugin_name . '-quizz',
			array( 'QuizzesController', 'manage' )
		);

		$hook_categories = add_submenu_page(
			$this->plugin_name . '-quizzes',
			apply_filters( $this->plugin_name . '-categories-page-title', esc_html__( 'Categorías', 'entusiastics-quiz-game' ) ),
			apply_filters( $this->plugin_name . '-categories-menu-title', esc_html__( 'Categorías', 'entusiastics-quiz-game' ) ),
			$entusiastics_caps,
			$this->plugin_name . '-categories',
			array( 'CategoriesTableController', 'render_quizzes_categories_list' )
		);

		add_action( "load-$hook_categories", array( $this, 'quizzes_categories_list_page_add_option') );

		add_submenu_page(
			NULL,
			apply_filters( $this->plugin_name . '-category-page-title', esc_html__( 'Categoría', 'entusiastics-quiz-game' ) ),
			apply_filters( $this->plugin_name . '-category-menu-title', esc_html__( 'Añadir/Editar Categoría', 'entusiastics-quiz-game' ) ),
			$entusiastics_caps,
			$this->plugin_name . '-category',
			array( 'CategoriesController', 'manage' )
		);

		$hook_profiles = add_submenu_page(
			$this->plugin_name . '-quizzes',
			apply_filters( $this->plugin_name . '-profiles-page-title', esc_html__( 'Listado Perfiles', 'entusiastics-quiz-game' ) ),
			apply_filters( $this->plugin_name . '-profiles-menu-title', esc_html__( 'Perfiles', 'entusiastics-quiz-game' ) ),
			$entusiastics_caps,
			$this->plugin_name . '-profiles',
			array( 'ProfilesTableController', 'render_profiles_list' )
		);

		add_action( "load-$hook_profiles", array( $this, 'profiles_list_page_add_option') );

		add_submenu_page(
			NULL,
			apply_filters( $this->plugin_name . '-profile-page-title', esc_html__( 'Añadir/Editar Perfil', 'entusiastics-quiz-game' ) ),
			apply_filters( $this->plugin_name . '-profile-menu-title', esc_html__( 'Añadir/Editar Perfil', 'entusiastics-quiz-game' ) ),
			$entusiastics_caps,
			$this->plugin_name . '-profile',
			array( 'ProfilesController', 'manage' )
		);

		add_submenu_page(
			NULL,
			apply_filters( $this->plugin_name . '-questions-page-title', esc_html__( 'Listado de Preguntas', 'entusiastics-quiz-game' ) ),
			apply_filters( $this->plugin_name . '-questions-menu-title', esc_html__( 'Preguntas', 'entusiastics-quiz-game' ) ),
			$entusiastics_caps,
			$this->plugin_name . '-questions',
			array( 'QuestionsController', 'manage' )
		);

		$hook_completed = add_submenu_page(
			$this->plugin_name . '-quizzes',
			apply_filters( $this->plugin_name . '-completed-list-page-title', esc_html__( 'Completados', 'entusiastics-quiz-game' ) ),
			apply_filters( $this->plugin_name . '-completed-list-menu-title', esc_html__( 'Completados', 'entusiastics-quiz-game' ) ),
			$entusiastics_caps,
			$this->plugin_name . '-completed-list',
			array( 'CompletedTableController', 'render_quizzes_completed_list' )
		);

		add_action( "load-$hook_completed", array( $this, 'completed_list_page_add_option') );

		$hook_completed_single = add_submenu_page(
			NULL,
			apply_filters( $this->plugin_name . '-completed-page-title', esc_html__( 'Ver Completado', 'entusiastics-quiz-game' ) ),
			apply_filters( $this->plugin_name . '-completed-menu-title', esc_html__( 'Ver Completado', 'entusiastics-quiz-game' ) ),
			$entusiastics_caps,
			$this->plugin_name . '-completed',
			array( 'CompletedController', 'manage' )
		);

		add_action( "load-$hook_completed_single", array( $this, 'completed_list_page_add_option') );

		add_submenu_page(
			$this->plugin_name . '-quizzes',
			apply_filters( $this->plugin_name . '-settings-page-title', esc_html__( 'Configuraciónes Entusiastics Game Plugin', 'entusiastics-quiz-game' ) ),
			apply_filters( $this->plugin_name . '-settings-menu-title', esc_html__( 'Configuración', 'entusiastics-quiz-game' ) ),
			$entusiastics_caps,
			$this->plugin_name . '-settings',
			array( $this, 'page_options' )
		);

		add_submenu_page(
			$this->plugin_name . '-quizzes',
			apply_filters( $this->plugin_name . '-help-page-title', esc_html__( 'Ayuda Entusiastics Game Plugin', 'entusiastics-quiz-game' ) ),
			apply_filters( $this->plugin_name . '-help-menu-title', esc_html__( 'Help', 'entusiastics-quiz-game' ) ),
			$entusiastics_caps,
			$this->plugin_name . '-help',
			array( $this, 'page_help' )
		);

	} // add_menu()

	/**
     * Manages any updates or upgrades needed before displaying notices.
     * Checks plugin version against version required for displaying
     * notices.
     */
	public function admin_notices_init() {

		$current_version = '1.0.0';

		if ( $this->version !== $current_version ) {

			// Do whatever upgrades needed here.

			update_option('1.0.0', $current_version);

			$this->add_notice();
		}

	} // admin_notices_init()

	/**
     * Manages any updates or upgrades needed before displaying notices.
     * Checks plugin version against version required for displaying
     * notices.
     */
	public function db_init() {

		add_action('init', array("EntusiasticsQuizGame", "init"));

	} // db_init()

	/**
	 * Quizzes list page options
	 *
	 * @return 	string 			Admin notices
	 */
	public function quizzes_list_page_add_option() {
		global $quizzes_list_table;

		$option = 'per_page';

		$args = array(
			'label' => __('Quizzes por pagina', 'entusiastics-quiz-game'),
			'default' => 5,
			'option' => 'quizzes_per_page'
		);

		add_screen_option( $option, $args );

		$quizzes_list_table = new QuizzesTableController();
	}

	/**
	 * Quizzes list page options
	 *
	 * @return 	string 			Admin notices
	 */
	public function profiles_list_page_add_option() {
		global $profiles_list_table;

		$option = 'per_page';

		$args = array(
			'label' => __('Perfiles por pagina', 'entusiastics-quiz-game'),
			'default' => 5,
			'option' => 'profiles_per_page'
		);

		add_screen_option( $option, $args );

		$profiles_list_table = new ProfilesTableController();
	}

	/**
	 * Completed list page options
	 *
	 * @return 	string 			Admin notices
	 */
	public function completed_list_page_add_option() {
		global $completed_list_table;

		$option = 'per_page';

		$args = array(
			'label' => __('Completados por pagina', 'entusiastics-quiz-game'),
			'default' => 5,
			'option' => 'completados_per_page'
		);

		add_screen_option( $option, $args );

		$completed_list_table = new CompletedTableController();
	}

	/**
	 * Quizzes Categories list page options
	 *
	 * @return 	string 			Admin notices
	 */
	public function quizzes_categories_list_page_add_option() {
		global $quizzes_list_table;

		$option = 'per_page';

		$args = array(
			'label' => __('Categorías por pagina', 'entusiastics-quiz-game'),
			'default' => 5,
			'option' => 'categories_per_page'
		);

		add_screen_option( $option, $args );

		$categories_list_table = new CategoriesTableController();
	}

	/**
	 * Displays admin notices
	 *
	 * @return 	string 			Admin notices
	 */
	public function display_admin_notices() {

		$notices = get_option( 'entusiastics_quiz_game_deferred_admin_notices' );

		if ( empty( $notices ) ) { return; }

		foreach ( $notices as $notice ) {

			echo '<div class="' . esc_attr( $notice['class'] ) . '"><p>' . $notice['notice'] . '</p></div>';

		}

		delete_option( 'entusiastics_quiz_game_deferred_admin_notices' );

    } // display_admin_notices()

	/**
	 * Register the stylesheets for the Dashboard.
	 *
	 * @since 		1.0.0
	 */
	public function enqueue_styles() {
		wp_enqueue_style( 'wp-color-picker' );
		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/' . $this->plugin_name . '-admin.css', array(), $this->version, 'all' );
		

	} // enqueue_styles()

	/**
	 * Register the JavaScript for the dashboard.
	 *
	 * @since 		1.0.0
	 */
	public function enqueue_scripts( $hook_suffix ) {
		wp_enqueue_script( 'jquery-ui-draggable' );
		wp_enqueue_script( 'jquery-ui-sortable' );
		wp_enqueue_script( $this->plugin_name.'-choice-js', plugin_dir_url( __FILE__ ) . 'js/' . $this->plugin_name . '-choice-form.js');
		wp_enqueue_script( $this->plugin_name.'-question-js', plugin_dir_url( __FILE__ ) . 'js/' . $this->plugin_name . '-question-form.js');
		wp_enqueue_script( $this->plugin_name.'-quiz-js', plugin_dir_url( __FILE__ ) . 'js/' . $this->plugin_name . '-quiz-form.js');
		wp_enqueue_script( $this->plugin_name.'-main-js', plugin_dir_url( __FILE__ ) . 'js/' . $this->plugin_name . '-main.js',array( 'wp-color-picker' ), false, true);

		
	} // enqueue_scripts()

	/**
	 * Register Media
	 *
	 * @since 		1.0.0
	 */
	public function load_media_files() {
		wp_enqueue_media();
	} // enqueue_scripts()	add_action( 'admin_enqueue_scripts', 'load_media_files' );


	/**
	 * Returns an array of options names, fields types, and default values
	 *
	 * @return 		array 			An array of options
	 */
	public static function get_options_list() {

		$options = array();
		$options[] = array( 'howtoapply', 'editor', '' );
		$options[] = array( 'entusiastics-quiz-game', 'text', '' );

		return $options;

	} // get_options_list()

	/**
	 * Adds links to the plugin links row
	 *
	 * @since 		1.0.0
	 * @param 		array 		$links 		The current array of row links
	 * @param 		string 		$file 		The name of the file
	 * @return 		array 					The modified array of row links
	 */
	public function link_row( $links, $file ) {

		if ( ENTUSIASTICS_QUIZ_GAME_FILE === $file ) {

			$links[] = '<a href="https://entusiastics.com">Entusiastics Web</a>';

		}

		return $links;

	} // link_row()

	/**
	 * Adds a link to the plugin settings page
	 *
	 * @since 		1.0.0
	 * @param 		array 		$links 		The current array of links
	 * @return 		array 					The modified array of links
	 */
	public function link_settings( $links ) {

		$links[] = sprintf( '<a href="%s">%s</a>', esc_url( admin_url( 'edit.php?post_type=quiz&page=' . $this->plugin_name . '-settings' ) ), esc_html__( 'Configuración', 'entusiastics-quiz-game' ) );

		return $links;

	} // link_settings()

	/**
	 * Creates the options page
	 *
	 * @since 		1.0.0
	 * @return 		void
	 */
	public function page_landing() {

		include( plugin_dir_path( __FILE__ ) . 'partials/' . $this->plugin_name . '-admin-landing.php' );

	} // page_options()


	/**
	 * Creates the options page
	 *
	 * @since 		1.0.0
	 * @return 		void
	 */
	public function page_options() {

		include( plugin_dir_path( __FILE__ ) . 'partials/' . $this->plugin_name . '-admin-page-settings.php' );

	} // page_options()

	/**
	 * Creates the help page
	 *
	 * @since 		1.0.0
	 * @return 		void
	 */
	public function page_help() {

		include( plugin_dir_path( __FILE__ ) . 'partials/' . $this->plugin_name . '-admin-page-help.php' );

	} // page_help()

	

	/**
	 * Registers settings fields with WordPress
	 */
	public function register_fields() {

		// add_settings_field( $id, $title, $callback, $menu_slug, $section, $args );

	} // register_fields()


	/**
	 * Registers plugin settings
	 *
	 * @since 		1.0.0
	 * @return 		void
	 */
	public function register_settings() {

		// register_setting( $option_group, $option_name, $sanitize_callback );

		register_setting(
			$this->plugin_name . '-options',
			$this->plugin_name . '-options',
			array( $this, 'validate_options' )
		);

	} // register_settings()

	private function sanitizer( $type, $data ) {

		if ( empty( $type ) ) { return; }
		if ( empty( $data ) ) { return; }

		$return 	= '';
		$sanitizer 	= new Quiz_Game_Sanitize();

		$sanitizer->set_data( $data );
		$sanitizer->set_type( $type );

		$return = $sanitizer->clean();

		unset( $sanitizer );

		return $return;

	} // sanitizer()


	/**
	 * Sets the class variable $options
	 */
	private function set_options() {

		$this->options = get_option( $this->plugin_name . '-options' );

	} // set_options()

	/**
	 * Validates saved options
	 *
	 * @since 		1.0.0
	 * @param 		array 		$input 			array of submitted plugin options
	 * @return 		array 						array of validated plugin options
	 */
	public function validate_options( $input ) {

		//wp_die( print_r( $input ) );

		$valid 		= array();
		$options 	= $this->get_options_list();

		foreach ( $options as $option ) {

			$name = $option[0];
			$type = $option[1];

			if ( 'repeater' === $type && is_array( $option[2] ) ) {

				$clean = array();

				foreach ( $option[2] as $field ) {

					foreach ( $input[$field[0]] as $data ) {

						if ( empty( $data ) ) { continue; }

						$clean[$field[0]][] = $this->sanitizer( $field[1], $data );

					} // foreach

				} // foreach

				$count = quiz_game_get_max( $clean );

				for ( $i = 0; $i < $count; $i++ ) {

					foreach ( $clean as $field_name => $field ) {

						$valid[$option[0]][$i][$field_name] = $field[$i];

					} // foreach $clean

				} // for

			} else {

				$valid[$option[0]] = $this->sanitizer( $type, $input[$name] );

			}

			/*if ( ! isset( $input[$option[0]] ) ) { continue; }

			$sanitizer = new Quiz_Game_Sanitize();

			$sanitizer->set_data( $input[$option[0]] );
			$sanitizer->set_type( $option[1] );

			$valid[$option[0]] = $sanitizer->clean();

			if ( $valid[$option[0]] != $input[$option[0]] ) {

				add_settings_error( $option[0], $option[0] . '_error', esc_html__( $option[0] . ' error.', 'entusiastics-quiz-game' ), 'error' );

			}

			unset( $sanitizer );*/

		}

		return $valid;

	} // validate_options()

} // class