<?php

/**
 * Quizz Category Model
 *
 *
 * @link       https://entusiastics.com
 * @since      1.0.0
 *
 * @package    Entusiastics_Quiz_Game
 * @subpackage Entusiastics_Quiz_Game/admin/models
 */

if( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

class Category {

	
	function add($vars) {

		global $wpdb;
		
		$name = sanitize_text_field($vars['name']);
		$category_description = sanitize_text_field($vars['category_description']);
		
		$result = $wpdb->query($wpdb->prepare("INSERT INTO ".ENTUSIASTICS_QUIZ_GAME_CATEGORIES." SET name=%s, category_description=%s", $name, $category_description));

		if($result === false) throw new Exception(__('DB Error', 'entusiastics-quiz-game'));

		return $wpdb->insert_id;

	} // end add
	
	function save($vars, $id) {
		global $wpdb;
		
		$vars['name'] = sanitize_text_field($vars['name']);	
		$vars['category_description'] = sanitize_text_field($vars['category_description']);
		
		$result = $wpdb->query($wpdb->prepare("UPDATE ".ENTUSIASTICS_QUIZ_GAME_CATEGORIES." SET
			name=%s, category_description=%s WHERE id=%d", 
			$vars['name'], strip_tags($vars['category_description']), intval($id)));	

		if($result === false) throw new Exception(__('DB Error', 'entusiastics-quiz-game'));

		return true;
	}
	
	function delete($category_id) {

		$result = $this->remove($category_id);

		entusiastics_quiz_game_redirect("admin.php?page=entusiastics-quiz-game-categories");
	}

	function bulkDelete($categories) {

		foreach ($categories as $category_id) {
			$this->remove($category_id);
		}

		entusiastics_quiz_game_redirect("admin.php?page=entusiastics-quiz-game-categories");

	} // end bulkDelete

	function remove($category_id) {

		global $wpdb;

		$id = intval($category_id);
		
		$_quiz = new Quiz();

		// update quizzes with this category to non categorized
		$quizzes_update = $_quiz->updateQuizCategoryToNonCategorized($id);
		
		// delete category
		$result = $wpdb->query($wpdb->prepare("DELETE FROM ".ENTUSIASTICS_QUIZ_GAME_CATEGORIES." WHERE id=%d", $id));
		
		if($result === false) throw new Exception(__('DB Error', 'entusiastics-quiz-game'));
		
		return true;
	} // end remove

	public function getCategoryByQuizzId() {
		global $wpdb;
		
		$sql = $wpdb->prepare("SELECT * FROM ".ENTUSIASTICS_QUIZ_GAME_CATEGORIES." WHERE id=%d", intval($_GET['id']));
		$category = $wpdb->get_row($sql);

		return $category;
	}

	public function getCategories() {
		global $wpdb;

		$order_by = 'name';
		
		$sql = $wpdb->prepare("SELECT * FROM ".ENTUSIASTICS_QUIZ_GAME_CATEGORIES." ORDER BY %s", $order_by);
		$categories = $wpdb->get_results($sql);

		return $categories;
	}  

} 